#!/usr/bin/env python3
# author: @netmanchris



# This section imports required libraries
import requests
import json

HEADERS = {'Accept': 'application/json', 'Content-Type':
    'application/json', 'Accept-encoding': 'application/json'}


headers = {'Accept': 'application/json', 'Content-Type':
    'application/json', 'Accept-encoding': 'application/json'}

def get_dev_alarms(devId, auth, url):
    """
    function takes the devId of a specific device and issues a RESTFUL call to get the current alarms for the target
    device.

    :param devId: int or str value of the target device

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return:list of dictionaries containing the alarms for this device

    :rtype: list

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.alarms import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> get_dev_alarms('10', auth.creds.auth.url)
    [{'OID': '1.3.6.1.6.3.1.1.5.3.0',
  'ackStatus': '1',
  'ackStatusDesc': 'Acknowledged',
  'ackTime': '1467854226',
  'ackTimeDesc': '2016-07-06 21:17:06',
  'ackUserName': 'admin',
  'alarmCategory': '3',
  'alarmCategoryDesc': 'Interface/Link Status Alarm',
  'alarmDesc': 'The interface GigabitEthernet2/0/23 is UP.',
  'alarmDetail': 'http://kontrolissues.thruhere.net:8086/imcrs/fault/alarm/160612',
  'alarmLevel': '5',
  'alarmLevelDesc': 'Info',
  'deviceId': '10',
  'deviceIp': '192.168.1.221',
  'deviceName': 'HP_5500EI',
  'faultTime': '1467765645',
  'faultTimeDesc': '2016-07-05 20:40:45',
  'holdInfo': '',
  'id': '160612',
  'originalType': '1',
  'originalTypeDesc': 'Trap',
  'paras': '*Interface Index=85;Interface Description=GigabitEthernet2/0/23;Interface Admin Status=1;Interface Operate Status=1',
  'parentId': '0',
  'recStatus': '1',
  'recStatusDesc': 'Recovered',
  'recTime': '1467765645',
  'recTimeDesc': '2016-07-05 20:40:45',
  'recUserName': '$SYSTEM',
  'remark': '',
  'somState': '0'},
 {'OID': '1.3.6.1.6.3.1.1.5.2.0',
  'ackStatus': '1',
  'ackStatusDesc': 'Acknowledged',
  'ackTime': '1467854226',
  'ackTimeDesc': '2016-07-06 21:17:06',
  'ackUserName': 'admin',
  'alarmCategory': '3',
  'alarmCategoryDesc': 'Interface/Link Status Alarm',
  'alarmDesc': 'The interface GigabitEthernet2/0/23 is DOWN.',
  'alarmDetail': 'http://kontrolissues.thruhere.net:8086/imcrs/fault/alarm/160539',
  'alarmLevel': '2',
  'alarmLevelDesc': 'Major',
  'deviceId': '10',
  'deviceIp': '192.168.1.221',
  'deviceName': 'HP_5500EI',
  'faultTime': '1467747889',
  'faultTimeDesc': '2016-07-05 15:44:49',
  'holdInfo': '',
  'id': '160539',
  'originalType': '1',
  'originalTypeDesc': 'Trap',
  'paras': '*Interface Index=85;Interface Description=GigabitEthernet2/0/23;Interface Admin Status=1;Interface Operate Status=2',
  'parentId': '0',
  'recStatus': '1',
  'recStatusDesc': 'Recovered',
  'recTime': '1467765645',
  'recTimeDesc': '2016-07-05 20:40:45',
  'recUserName': '$SYSTEM',
  'remark': '',
  'somState': '0'},
 {'OID': '1.3.6.1.6.3.1.1.5.2.0',
  'ackStatus': '1',
  'ackStatusDesc': 'Acknowledged',
  'ackTime': '1467854226',
  'ackTimeDesc': '2016-07-06 21:17:06',
  'ackUserName': 'admin',
  'alarmCategory': '3',
  'alarmCategoryDesc': 'Interface/Link Status Alarm',
  'alarmDesc': 'The interface GigabitEthernet2/0/20 is DOWN.',
  'alarmDetail': 'http://kontrolissues.thruhere.net:8086/imcrs/fault/alarm/160524',
  'alarmLevel': '2',
  'alarmLevelDesc': 'Major',
  'deviceId': '10',
  'deviceIp': '192.168.1.221',
  'deviceName': 'HP_5500EI',
  'faultTime': '1467747461',
  'faultTimeDesc': '2016-07-05 15:37:41',
  'holdInfo': '',
  'id': '160524',
  'originalType': '1',
  'originalTypeDesc': 'Trap',
  'paras': '*Interface Index=82;Interface Description=GigabitEthernet2/0/20;Interface Admin Status=1;Interface Operate Status=2',
  'parentId': '0',
  'recStatus': '1',
  'recStatusDesc': 'Recovered',
  'recTime': '1467854218',
  'recTimeDesc': '2016-07-06 21:16:58',
  'recUserName': 'admin',
  'remark': '',
  'somState': '0'},
 {'OID': '1.3.6.1.6.3.1.1.5.2.0',
  'ackStatus': '1',
  'ackStatusDesc': 'Acknowledged',
  'ackTime': '1467854226',
  'ackTimeDesc': '2016-07-06 21:17:06',
  'ackUserName': 'admin',
  'alarmCategory': '3',
  'alarmCategoryDesc': 'Interface/Link Status Alarm',
  'alarmDesc': 'The interface GigabitEthernet1/0/21 is DOWN.',
  'alarmDetail': 'http://kontrolissues.thruhere.net:8086/imcrs/fault/alarm/160496',
  'alarmLevel': '2',
  'alarmLevelDesc': 'Major',
  'deviceId': '10',
  'deviceIp': '192.168.1.221',
  'deviceName': 'HP_5500EI',
  'faultTime': '1467745458',
  'faultTimeDesc': '2016-07-05 15:04:18',
  'holdInfo': '',
  'id': '160496',
  'originalType': '1',
  'originalTypeDesc': 'Trap',
  'paras': '*Interface Index=21;Interface Description=GigabitEthernet1/0/21;Interface Admin Status=1;Interface Operate Status=2',
  'parentId': '0',
  'recStatus': '1',
  'recStatusDesc': 'Recovered',
  'recTime': '1467854218',
  'recTimeDesc': '2016-07-06 21:16:58',
  'recUserName': 'admin',
  'remark': '',
  'somState': '0'},
 {'OID': '1.3.6.1.6.3.1.1.5.3.0',
  'ackStatus': '1',
  'ackStatusDesc': 'Acknowledged',
  'ackTime': '1467854226',
  'ackTimeDesc': '2016-07-06 21:17:06',
  'ackUserName': 'admin',
  'alarmCategory': '3',
  'alarmCategoryDesc': 'Interface/Link Status Alarm',
  'alarmDesc': 'The interface GigabitEthernet2/0/20 is UP.',
  'alarmDetail': 'http://kontrolissues.thruhere.net:8086/imcrs/fault/alarm/160487',
  'alarmLevel': '5',
  'alarmLevelDesc': 'Info',
  'deviceId': '10',
  'deviceIp': '192.168.1.221',
  'deviceName': 'HP_5500EI',
  'faultTime': '1467742807',
  'faultTimeDesc': '2016-07-05 14:20:07',
  'holdInfo': '',
  'id': '160487',
  'originalType': '1',
  'originalTypeDesc': 'Trap',
  'paras': '*Interface Index=82;Interface Description=GigabitEthernet2/0/20;Interface Admin Status=1;Interface Operate Status=1',
  'parentId': '0',
  'recStatus': '1',
  'recStatusDesc': 'Recovered',
  'recTime': '1467742807',
  'recTimeDesc': '2016-07-05 14:20:07',
  'recUserName': '$SYSTEM',
  'remark': '',
  'somState': '0'},
 {'OID': '1.3.6.1.6.3.1.1.5.2.0',
  'ackStatus': '1',
  'ackStatusDesc': 'Acknowledged',
  'ackTime': '1467854226',
  'ackTimeDesc': '2016-07-06 21:17:06',
  'ackUserName': 'admin',
  'alarmCategory': '3',
  'alarmCategoryDesc': 'Interface/Link Status Alarm',
  'alarmDesc': 'The interface GigabitEthernet2/0/19 is DOWN.',
  'alarmDetail': 'http://kontrolissues.thruhere.net:8086/imcrs/fault/alarm/160485',
  'alarmLevel': '2',
  'alarmLevelDesc': 'Major',
  'deviceId': '10',
  'deviceIp': '192.168.1.221',
  'deviceName': 'HP_5500EI',
  'faultTime': '1467742775',
  'faultTimeDesc': '2016-07-05 14:19:35',
  'holdInfo': '',
  'id': '160485',
  'originalType': '1',
  'originalTypeDesc': 'Trap',
  'paras': '*Interface Index=81;Interface Description=GigabitEthernet2/0/19;Interface Admin Status=1;Interface Operate Status=2',
  'parentId': '0',
  'recStatus': '1',
  'recStatusDesc': 'Recovered',
  'recTime': '1467854218',
  'recTimeDesc': '2016-07-06 21:16:58',
  'recUserName': 'admin',
  'remark': '',
  'somState': '0'},
 {'OID': '1.3.6.1.6.3.1.1.5.3.0',
  'ackStatus': '1',
  'ackStatusDesc': 'Acknowledged',
  'ackTime': '1467854226',
  'ackTimeDesc': '2016-07-06 21:17:06',
  'ackUserName': 'admin',
  'alarmCategory': '3',
  'alarmCategoryDesc': 'Interface/Link Status Alarm',
  'alarmDesc': 'The interface GigabitEthernet2/0/19 is UP.',
  'alarmDetail': 'http://kontrolissues.thruhere.net:8086/imcrs/fault/alarm/160484',
  'alarmLevel': '5',
  'alarmLevelDesc': 'Info',
  'deviceId': '10',
  'deviceIp': '192.168.1.221',
  'deviceName': 'HP_5500EI',
  'faultTime': '1467742742',
  'faultTimeDesc': '2016-07-05 14:19:02',
  'holdInfo': '',
  'id': '160484',
  'originalType': '1',
  'originalTypeDesc': 'Trap',
  'paras': '*Interface Index=81;Interface Description=GigabitEthernet2/0/19;Interface Admin Status=1;Interface Operate Status=1',
  'parentId': '0',
  'recStatus': '1',
  'recStatusDesc': 'Recovered',
  'recTime': '1467742742',
  'recTimeDesc': '2016-07-05 14:19:02',
  'recUserName': '$SYSTEM',
  'remark': '',
  'somState': '0'},
 {'OID': '1.3.6.1.6.3.1.1.5.3.0',
  'ackStatus': '1',
  'ackStatusDesc': 'Acknowledged',
  'ackTime': '1467854226',
  'ackTimeDesc': '2016-07-06 21:17:06',
  'ackUserName': 'admin',
  'alarmCategory': '3',
  'alarmCategoryDesc': 'Interface/Link Status Alarm',
  'alarmDesc': 'The interface GigabitEthernet1/0/21 is UP.',
  'alarmDetail': 'http://kontrolissues.thruhere.net:8086/imcrs/fault/alarm/160447',
  'alarmLevel': '5',
  'alarmLevelDesc': 'Info',
  'deviceId': '10',
  'deviceIp': '192.168.1.221',
  'deviceName': 'HP_5500EI',
  'faultTime': '1467736157',
  'faultTimeDesc': '2016-07-05 12:29:17',
  'holdInfo': '',
  'id': '160447',
  'originalType': '1',
  'originalTypeDesc': 'Trap',
  'paras': '*Interface Index=21;Interface Description=GigabitEthernet1/0/21;Interface Admin Status=1;Interface Operate Status=1',
  'parentId': '0',
  'recStatus': '1',
  'recStatusDesc': 'Recovered',
  'recTime': '1467736157',
  'recTimeDesc': '2016-07-05 12:29:17',
  'recUserName': '$SYSTEM',
  'remark': '',
  'somState': '0'},
 {'OID': '1.3.6.1.6.3.1.1.5.2.0',
  'ackStatus': '1',
  'ackStatusDesc': 'Acknowledged',
  'ackTime': '1467854226',
  'ackTimeDesc': '2016-07-06 21:17:06',
  'ackUserName': 'admin',
  'alarmCategory': '3',
  'alarmCategoryDesc': 'Interface/Link Status Alarm',
  'alarmDesc': 'The interface GigabitEthernet1/0/21 is DOWN.',
  'alarmDetail': 'http://kontrolissues.thruhere.net:8086/imcrs/fault/alarm/160446',
  'alarmLevel': '2',
  'alarmLevelDesc': 'Major',
  'deviceId': '10',
  'deviceIp': '192.168.1.221',
  'deviceName': 'HP_5500EI',
  'faultTime': '1467736125',
  'faultTimeDesc': '2016-07-05 12:28:45',
  'holdInfo': '',
  'id': '160446',
  'originalType': '1',
  'originalTypeDesc': 'Trap',
  'paras': '*Interface Index=21;Interface Description=GigabitEthernet1/0/21;Interface Admin Status=1;Interface Operate Status=2',
  'parentId': '160438',
  'recStatus': '1',
  'recStatusDesc': 'Recovered',
  'recTime': '1467736157',
  'recTimeDesc': '2016-07-05 12:29:17',
  'recUserName': '$SYSTEM',
  'remark': '',
  'somState': '0'},
 {'OID': '1.3.6.1.6.3.1.1.5.3.0',
  'ackStatus': '1',
  'ackStatusDesc': 'Acknowledged',
  'ackTime': '1467854226',
  'ackTimeDesc': '2016-07-06 21:17:06',
  'ackUserName': 'admin',
  'alarmCategory': '3',
  'alarmCategoryDesc': 'Interface/Link Status Alarm',
  'alarmDesc': 'The interface GigabitEthernet1/0/21 is UP.',
  'alarmDetail': 'http://kontrolissues.thruhere.net:8086/imcrs/fault/alarm/160444',
  'alarmLevel': '5',
  'alarmLevelDesc': 'Info',
  'deviceId': '10',
  'deviceIp': '192.168.1.221',
  'deviceName': 'HP_5500EI',
  'faultTime': '1467736092',
  'faultTimeDesc': '2016-07-05 12:28:12',
  'holdInfo': '',
  'id': '160444',
  'originalType': '1',
  'originalTypeDesc': 'Trap',
  'paras': '*Interface Index=21;Interface Description=GigabitEthernet1/0/21;Interface Admin Status=1;Interface Operate Status=1',
  'parentId': '0',
  'recStatus': '1',
  'recStatusDesc': 'Recovered',
  'recTime': '1467736092',
  'recTimeDesc': '2016-07-05 12:28:12',
  'recUserName': '$SYSTEM',
  'remark': '',
  'somState': '0'}]
    """
    # checks to see if the imc credentials are already available
    get_dev_alarm_url = "/imcrs/fault/alarm?operatorName=admin&deviceId=" + \
                        str(devId) + "&desc=false"
    f_url = url + get_dev_alarm_url
    # creates the URL using the payload variable as the contents
    r = requests.get(f_url, auth=auth, headers=headers)
    try:
        if r.status_code == 200:
            dev_alarm = (json.loads(r.text))
            if 'alarm' in dev_alarm:
                return dev_alarm['alarm']
            else:
                return "Device has no alarms"
    except requests.exceptions.RequestException as e:
            return "Error:\n" + str(e) + ' get_dev_alarms: An Error has occured'


def get_realtime_alarm(username, auth, url):
    """Takes in no param as input to fetch RealTime Alarms from HP IMC RESTFUL API

    :param username OpeatorName, String type. Required. Default Value "admin". Checks the operator
     has the privileges to view the Real-Time Alarms.

    :param devId: int or str value of the target device

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return:list of dictionaries where each element of the list represents a single alarm as pulled from the the current
     list of realtime alarms in the HPE IMC Platform

    :rtype: list

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.alarms import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> get_realtime_alarm('alarm', auth.creds, auth.url)
    [ {'deviceDisplay': '10.10.10.5',
  'faultDesc': 'The device(MAC Address:   F4 B7 E2 95 A0 CD) is detected to be flooding the network. Attack Frame Type: Probe Request.',
  'faultTime': '1469422319',
  'id': '185253',
  'severity': '3',
  'userAckType': '0',
  'userAckUserName': ''},
 {'deviceDisplay': '10.10.10.5',
  'faultDesc': 'The device(MAC Address:   F4 B7 E2 95 A0 CD) is detected to be flooding the network. Attack Frame Type: Probe Request.',
  'faultTime': '1469421710',
  'id': '185202',
  'severity': '3',
  'userAckType': '0',
  'userAckUserName': ''}]


    """
    get_realtime_alarm_url = "/imcrs/fault/faultRealTime?operatorName=" + username
    f_url = url + get_realtime_alarm_url
    print (f_url)
    r = requests.get(f_url, auth=auth, headers=headers)
    try:
        print (r.status_code)
        realtime_alarm_list = (json.loads(r.text))
        return realtime_alarm_list['faultRealTime']['faultRealTimeList']
    except requests.exceptions.RequestException as e:
        return "Error:\n" + str(e) + ' get_realtime_alarm: An Error has occured'


def get_alarms(username, auth, url):
    """Takes in no param as input to fetch RealTime Alarms from HP IMC RESTFUL API

    :param username OpeatorName, String type. Required. Default Value "admin". Checks the operator
     has the privileges to view the Real-Time Alarms.

    :param devId: int or str value of the target device

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return:list of dictionaries where each element of the list represents a single alarm as pulled from the the current
     list of browse alarms in the HPE IMC Platform

    :rtype: list

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.alarms import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>>get_alarms('admin', auth.creds, auth.url)

    >>>get_alarms('admin', auth.creds, auth.url)
    [{'OID': '1.3.6.1.4.1.11.2.14.11.15.2.75.5.3.1.6.7',
  'ackStatus': '0',
  'ackStatusDesc': 'Unacknowledged',
  'ackTime': '0',
  'ackTimeDesc': '',
  'ackUserName': '',
  'alarmCategory': '1001',
  'alarmCategoryDesc': 'Wireless Device Alarm',
  'alarmDesc': 'The device(MAC Address:   F4 B7 E2 95 A0 CD) is detected to be flooding the network. Attack Frame Type: Probe Request.',
  'alarmDetail': 'http://10.101.0.203:8080/imcrs/fault/alarm/187616',
  'alarmLevel': '3',
  'alarmLevelDesc': 'Minor',
  'deviceId': '31',
  'deviceIp': '10.10.10.5',
  'deviceName': 'HP830_WSC',
  'faultTime': '1469471298',
  'faultTimeDesc': '2016-07-25 14:28:18',
  'holdInfo': '',
  'id': '187616',
  'originalType': '1',
  'originalTypeDesc': 'Trap',
  'paras': '*Attack MAC=  F4 B7 E2 95 A0 CD;Attack Frame Type=Probe Request',
  'parentId': '0',
  'recStatus': '0',
  'recStatusDesc': 'Unrecovered',
  'recTime': '0',
  'recTimeDesc': '',
  'recUserName': '',
  'remark': '',
  'somState': '0'},
 {'OID': '1.3.6.1.4.1.25506.4.2.2.2.6.1',
  'ackStatus': '0',
  'ackStatusDesc': 'Unacknowledged',
  'ackTime': '0',
  'ackTimeDesc': '',
  'ackUserName': '',
  'alarmCategory': '15',
  'alarmCategoryDesc': 'Other Alarms from NMS',
  'alarmDesc': 'iMC alarm system has received 100 alarms about the event (hh3cPeriodicalTrap) of device ADP_Initial_Config(10.20.10.10) from 2016-07-25 12:56:01 to 2016-07-25 14:36:01.',
  'alarmDetail': 'http://10.101.0.203:8080/imcrs/fault/alarm/187625',
  'alarmLevel': '4',
  'alarmLevelDesc': 'Warning',
  'deviceId': '0',
  'deviceIp': '127.0.0.1',
  'deviceName': 'NMS',
  'faultTime': '1469471761',
  'faultTimeDesc': '2016-07-25 14:36:01',
  'id': '187625',
  'originalType': '3',
  'originalTypeDesc': 'iMC',
  'paras': 'Start Time=2016-07-25 12:56:01;Stop Time=2016-07-25 14:36:01;Times=100;*Repeat Event Name=hh3cPeriodicalTrap;*Device IP=10.20.10.10;Device Name=ADP_Initial_Config',
  'parentId': '173953',
  'recStatus': '0',
  'recStatusDesc': 'Unrecovered',
  'recTime': '0',
  'recTimeDesc': '',
  'recUserName': '',
  'remark': '',
  'somState': '0'},
 {'OID': '1.3.6.1.4.1.25506.4.2.2.2.6.1',
  'ackStatus': '0',
  'ackStatusDesc': 'Unacknowledged',
  'ackTime': '0',
  'ackTimeDesc': '',
  'ackUserName': '',
  'alarmCategory': '15',
  'alarmCategoryDesc': 'Other Alarms from NMS',
  'alarmDesc': 'iMC alarm system has received 100 alarms about the event (Some ambient device interferes) of device HP830_WSC(10.10.10.5) from 2016-07-25 14:07:06 to 2016-07-25 14:40:21.',
  'alarmDetail': 'http://10.101.0.203:8080/imcrs/fault/alarm/187626',
  'alarmLevel': '4',
  'alarmLevelDesc': 'Warning',
  'deviceId': '0',
  'deviceIp': '127.0.0.1',
  'deviceName': 'NMS',
  'faultTime': '1469472021',
  'faultTimeDesc': '2016-07-25 14:40:21',
  'id': '187626',
  'originalType': '3',
  'originalTypeDesc': 'iMC',
  'paras': 'Start Time=2016-07-25 14:07:06;Stop Time=2016-07-25 14:40:21;Times=100;*Repeat Event Name=Some ambient device interferes;*Device IP=10.10.10.5;Device Name=HP830_WSC',
  'parentId': '173953',
  'recStatus': '0',
  'recStatusDesc': 'Unrecovered',
  'recTime': '0',
  'recTimeDesc': '',
  'recUserName': '',
  'remark': '',
  'somState': '0'},
 {'OID': '1.3.6.1.4.1.25506.4.2.3.2.6.11',
  'ackStatus': '0',
  'ackStatusDesc': 'Unacknowledged',
  'ackTime': '0',
  'ackTimeDesc': '',
  'ackUserName': '',
  'alarmCategory': '9',
  'alarmCategoryDesc': 'Traffic Analysis and Audit Alarm',
  'alarmDesc': 'The in direction of interface GigabitEthernet1/0/23 on device 192.168.1.221 at 2016-07-25 14:32:00 is greater than the baseline(2.87 Mbps).',
  'alarmDetail': 'http://10.101.0.203:8080/imcrs/fault/alarm/187631',
  'alarmLevel': '2',
  'alarmLevelDesc': 'Major',
  'deviceId': '67',
  'deviceIp': '10.101.0.203',
  'deviceName': 'HPEIMC72WIN2K12R2',
  'faultTime': '1469472140',
  'faultTimeDesc': '2016-07-25 14:42:20',
  'holdInfo': '',
  'id': '187631',
  'originalType': '3',
  'originalTypeDesc': 'iMC',
  'paras': '*UNBA Server Name=NMS;*UNBA Server IP=127.0.0.1;*Alarm Object Name=The in direction of interface GigabitEthernet1/0/23 on device 192.168.1.221;Alarm Occur Time=2016-07-25 14:32:00;Current Flow Speed=48.38 Mbps;Alarm Severity=2;Threshold Speed=2.87 Mbps;Device IP=192.168.1.221;Interface Index=23;Interface Name=GigabitEthernet1/0/23;Flow Direction=1',
  'parentId': '0',
  'recStatus': '0',
  'recStatusDesc': 'Unrecovered',
  'recTime': '0',
  'recTimeDesc': '',
  'recUserName': '',
  'remark': '',
  'somState': '0'},
 {'OID': '1.3.6.1.4.1.25506.4.2.3.2.6.11',
  'ackStatus': '0',
  'ackStatusDesc': 'Unacknowledged',
  'ackTime': '0',
  'ackTimeDesc': '',
  'ackUserName': '',
  'alarmCategory': '9',
  'alarmCategoryDesc': 'Traffic Analysis and Audit Alarm',
  'alarmDesc': 'The out direction of interface GigabitEthernet1/0/2 on device 192.168.1.221 at 2016-07-25 14:32:00 is greater than the baseline(1.78 Mbps).',
  'alarmDetail': 'http://10.101.0.203:8080/imcrs/fault/alarm/187632',
  'alarmLevel': '2',
  'alarmLevelDesc': 'Major',
  'deviceId': '67',
  'deviceIp': '10.101.0.203',
  'deviceName': 'HPEIMC72WIN2K12R2',
  'faultTime': '1469472140',
  'faultTimeDesc': '2016-07-25 14:42:20',
  'holdInfo': '',
  'id': '187632',
  'originalType': '3',
  'originalTypeDesc': 'iMC',
  'paras': '*UNBA Server Name=NMS;*UNBA Server IP=127.0.0.1;*Alarm Object Name=The out direction of interface GigabitEthernet1/0/2 on device 192.168.1.221;Alarm Occur Time=2016-07-25 14:32:00;Current Flow Speed=48.79 Mbps;Alarm Severity=2;Threshold Speed=1.78 Mbps;Device IP=192.168.1.221;Interface Index=2;Interface Name=GigabitEthernet1/0/2;Flow Direction=0',
  'parentId': '0',
  'recStatus': '0',
  'recStatusDesc': 'Unrecovered',
  'recTime': '0',
  'recTimeDesc': '',
  'recUserName': '',
  'remark': '',
  'somState': '0'},
 {'OID': '1.3.6.1.4.1.25506.4.2.2.2.6.1',
  'ackStatus': '0',
  'ackStatusDesc': 'Unacknowledged',
  'ackTime': '0',
  'ackTimeDesc': '',
  'ackUserName': '',
  'alarmCategory': '15',
  'alarmCategoryDesc': 'Other Alarms from NMS',
  'alarmDesc': 'iMC alarm system has received 100 alarms about the event (Some ambient AP interferes) of device HP830_WSC(10.10.10.5) from 2016-07-25 14:14:00 to 2016-07-25 14:47:55.',
  'alarmDetail': 'http://10.101.0.203:8080/imcrs/fault/alarm/187637',
  'alarmLevel': '4',
  'alarmLevelDesc': 'Warning',
  'deviceId': '0',
  'deviceIp': '127.0.0.1',
  'deviceName': 'NMS',
  'faultTime': '1469472475',
  'faultTimeDesc': '2016-07-25 14:47:55',
  'id': '187637',
  'originalType': '3',
  'originalTypeDesc': 'iMC',
  'paras': 'Start Time=2016-07-25 14:14:00;Stop Time=2016-07-25 14:47:55;Times=100;*Repeat Event Name=Some ambient AP interferes;*Device IP=10.10.10.5;Device Name=HP830_WSC',
  'parentId': '173953',
  'recStatus': '0',
  'recStatusDesc': 'Unrecovered',
  'recTime': '0',
  'recTimeDesc': '',
  'recUserName': '',
  'remark': '',
  'somState': '0'},
 {'OID': '1.3.6.1.4.1.25506.4.2.2.2.6.1',
  'ackStatus': '0',
  'ackStatusDesc': 'Unacknowledged',
  'ackTime': '0',
  'ackTimeDesc': '',
  'ackUserName': '',
  'alarmCategory': '15',
  'alarmCategoryDesc': 'Other Alarms from NMS',
  'alarmDesc': 'iMC alarm system has received 100 alarms about the event (Some ambient device interferes) of device HP830_WSC(10.10.10.5) from 2016-07-25 14:40:21 to 2016-07-25 15:15:50.',
  'alarmDetail': 'http://10.101.0.203:8080/imcrs/fault/alarm/187654',
  'alarmLevel': '4',
  'alarmLevelDesc': 'Warning',
  'deviceId': '0',
  'deviceIp': '127.0.0.1',
  'deviceName': 'NMS',
  'faultTime': '1469474150',
  'faultTimeDesc': '2016-07-25 15:15:50',
  'id': '187654',
  'originalType': '3',
  'originalTypeDesc': 'iMC',
  'paras': 'Start Time=2016-07-25 14:40:21;Stop Time=2016-07-25 15:15:50;Times=100;*Repeat Event Name=Some ambient device interferes;*Device IP=10.10.10.5;Device Name=HP830_WSC',
  'parentId': '173953',
  'recStatus': '0',
  'recStatusDesc': 'Unrecovered',
  'recTime': '0',
  'recTimeDesc': '',
  'recUserName': '',
  'remark': '',
  'somState': '0'}]
    """
    get_alarms_url = "/imcrs/fault/alarm?operatorName=" + username + "&recStatus=0&ackStatus=0&timeRange=0&size=50&desc=true"
    f_url = url + get_alarms_url
    r = requests.get(f_url, auth=auth, headers=headers)
    # r.status_code
    try:
        if r.status_code == 200:
            alarm_list = (json.loads(r.text))
            return alarm_list['alarm']
    except requests.exceptions.RequestException as e:
            return "Error:\n" + str(e) + ' get_alarms: An Error has occured'






#!/usr/bin/env python3
# author: @netmanchris



# This section imports required libraries
import requests
import json
from pyhpeimc.auth import IMCAuth

HEADERS = {'Accept': 'application/json', 'Content-Type':
    'application/json', 'Accept-encoding': 'application/json'}

#auth = IMCAuth('http://','10.101.0.201','8080', 'admin','admin')

headers = {'Accept': 'application/json', 'Content-Type':
    'application/json', 'Accept-encoding': 'application/json'}

def add_perf_task(task, auth, url):
    """
        function takes the a python dict containing all necessary fields for a performance tasks, transforms the dict into
         JSON and issues a RESTFUL call to create the performance task.
        device.

        :param task: dictionary containing all required fields for performance tasks

        :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

        :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

        :return: 204

        :rtype: str

        >>> from pyhpeimc.auth import *

        >>> from pyhpeimc.plat.perf import *

        >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

        >>> new_task = {'indexDesc': '1.3.6.1.4.1.9.9.13.1.3.1.3',
 'indexType': '[index1[0]:ciscoEnvMonTemperatureStatusValue:1:0]',
 'itemFunction': '1.3.6.1.4.1.9.9.13.1.3.1.3',
 'itemName': 'Cisco_Temperature',
 'selectDefaultUnit': '400',
 'unit': 'Celsius'}

        >>> add_perf_task(new_task, auth.creds, auth.url)
        """
    add_perf_task_url = "/imcrs/perf/task"
    f_url = url + add_perf_task_url
    payload = json.dumps(task)
    # creates the URL using the payload variable as the contents
    r = requests.post(f_url, data = payload, auth=auth, headers=headers)
    try:
        return r.status_code

    except requests.exceptions.RequestException as e:
        return "Error:\n" + str(e) + ' get_dev_alarms: An Error has occured'



#!/usr/bin/env python3
# author: @netmanchris



# This section imports required libraries
import json
import requests
import ipaddress


HEADERS = {'Accept': 'application/json', 'Content-Type':
    'application/json', 'Accept-encoding': 'application/json'}





def get_real_time_locate(ipAddress, auth, url):
    """
    function takes the ipAddress of a specific host and issues a RESTFUL call to get the device and interface that the
    target host is currently connected to. Note: Although intended to return a single location, Multiple locations may
    be returned for a single host due to a partially discovered network or misconfigured environment.

    :param ipAddress: str value valid IPv4 IP address

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return: list of dictionaries where each element of the list represents the location of the target host

    :rtype: list

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.termaccess import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> get_real_time_locate('10.101.0.51', auth.creds, auth.url)
    [{'deviceId': '10',
  'deviceIp': '192.168.1.221',
  'ifDesc': 'GigabitEthernet1/0/2',
  'ifIndex': '2',
  'locateIp': '10.101.0.51'}]


    """
    real_time_locate_url = "/imcrs/res/access/realtimeLocate?type=2&value=" + str(ipAddress) + "&total=false"
    f_url = url + real_time_locate_url
    r = requests.get(f_url, auth=auth, headers=HEADERS)  # creates the URL using the payload variable as the contents
    try:
        if r.status_code == 200:
            response =  json.loads(r.text)
            if 'realtimeLocation' in response:
                real_time_locate = json.loads(r.text)['realtimeLocation']
                if type(real_time_locate) is dict:
                    real_time_locate = [real_time_locate]
                    return real_time_locate
                else:
                    return json.loads(r.text)['realtimeLocation']
            else:
                return json.loads(r.text)

    except requests.exceptions.RequestException as e:
            return "Error:\n" + str(e) + " get_real_time_locate: An Error has occured"


def get_ip_mac_arp_list(devId, auth,url):
    """
    function takes devid of specific device and issues a RESTFUL call to get the IP/MAC/ARP list from the target device.

    :param devId: int or str value of the target device.

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return: list of dictionaries containing the IP/MAC/ARP list of the target device.

    :rtype: list

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.termaccess import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> get_ip_mac_arp_list('10', auth.creds, auth.url)
    [{'device': 'http://kontrolissues.thruhere.net:8086/imcrs/plat/res/device/10',
  'deviceId': '10',
  'deviceIp': '10.10.10.5',
  'ifIndex': '39',
  'iface': 'http://kontrolissues.thruhere.net:8086/imcrs/plat/res/device/10/interface/39',
  'macAddress': 'd0:7e:28:80:40:0c'},
 {'device': 'http://kontrolissues.thruhere.net:8086/imcrs/plat/res/device/10',
  'deviceId': '10',
  'deviceIp': '10.10.3.3',
  'ifIndex': '51',
  'iface': 'http://kontrolissues.thruhere.net:8086/imcrs/plat/res/device/10/interface/51',
  'macAddress': 'd0:7e:28:80:40:0c'},
 {'device': 'http://kontrolissues.thruhere.net:8086/imcrs/plat/res/device/10',
  'deviceId': '10',
  'deviceIp': '10.10.3.5',
  'ifIndex': '51',
  'iface': 'http://kontrolissues.thruhere.net:8086/imcrs/plat/res/device/10/interface/51',
  'macAddress': 'd0:7e:28:80:40:0f'},
 {'device': 'http://kontrolissues.thruhere.net:8086/imcrs/plat/res/device/10',
  'deviceId': '10',
  'deviceIp': '10.101.0.1',
  'ifIndex': '30',
  'iface': 'http://kontrolissues.thruhere.net:8086/imcrs/plat/res/device/10/interface/30',
  'macAddress': '00:1b:d4:47:1e:68'},
 {'device': 'http://kontrolissues.thruhere.net:8086/imcrs/plat/res/device/10',
  'deviceId': '10',
  'deviceIp': '10.101.0.100',
  'ifIndex': '30',
  'iface': 'http://kontrolissues.thruhere.net:8086/imcrs/plat/res/device/10/interface/30',
  'macAddress': '00:50:56:66:7d:73'},
 {'device': 'http://kontrolissues.thruhere.net:8086/imcrs/plat/res/device/10',
  'deviceId': '10',
  'deviceIp': '10.101.0.102',
  'ifIndex': '30',
  'iface': 'http://kontrolissues.thruhere.net:8086/imcrs/plat/res/device/10/interface/30',
  'macAddress': '14:99:e2:28:ec:24'},
 {'device': 'http://kontrolissues.thruhere.net:8086/imcrs/plat/res/device/10',
  'deviceId': '10',
  'deviceIp': '10.101.0.105',
  'ifIndex': '30',
  'iface': 'http://kontrolissues.thruhere.net:8086/imcrs/plat/res/device/10/interface/30',
  'macAddress': '78:48:59:49:18:c0'},
 {'device': 'http://kontrolissues.thruhere.net:8086/imcrs/plat/res/device/10',
  'deviceId': '10',
  'deviceIp': '10.101.0.107',
  'ifIndex': '30',
  'iface': 'http://kontrolissues.thruhere.net:8086/imcrs/plat/res/device/10/interface/30',
  'macAddress': '78:48:59:49:16:40'},
 {'device': 'http://kontrolissues.thruhere.net:8086/imcrs/plat/res/device/10',
  'deviceId': '10',
  'deviceIp': '10.101.0.108',
  'ifIndex': '30',
  'iface': 'http://kontrolissues.thruhere.net:8086/imcrs/plat/res/device/10/interface/30',
  'macAddress': '2c:41:38:7f:a8:eb'},
 {'device': 'http://kontrolissues.thruhere.net:8086/imcrs/plat/res/device/10',
  'deviceId': '10',
  'deviceIp': '10.101.0.110',
  'ifIndex': '30',
  'iface': 'http://kontrolissues.thruhere.net:8086/imcrs/plat/res/device/10/interface/30',
  'macAddress': 'c0:cb:38:63:42:cd'}]

    """
    if auth is None or url is None:  # checks to see if the imc credentials are already available
        set_imc_creds()
    ip_mac_arp_list_url = "/imcrs/res/access/ipMacArp/" + str(devId)
    f_url = url + ip_mac_arp_list_url
    r = requests.get(f_url, auth=auth, headers=HEADERS)  # creates the URL using the payload variable as the contents
    try:
        if r.status_code == 200:
            macarplist = (json.loads(r.text))
            if len(macarplist) > 1:
                return macarplist['ipMacArp']
            else:
                return ['this function is unsupported']

    except requests.exceptions.RequestException as e:
            return "Error:\n" + str(e) + " get_ip_mac_arp_list: An Error has occured"


#this section deals with the IP Address Manager functions with terminal access of HPE IMC Base platform

def get_ip_scope(auth, url, scopeId=None,):
    """
    function requires no inputs and returns all IP address scopes currently configured on the HPE IMC server. If the
    optional scopeId parameter is included, this will automatically return only the desired scope id.

    :param scopeId: integer of the desired scope id ( optional )

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return: list of dictionary objects where each element of the list represents one IP scope

    :rtype: list

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.termaccess import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> get_ip_scope(auth.creds, auth.url)
    [{'description': 'Allocated IP addresses that do not belong to any allocated IP segment.',
  'id': '-299',
  'ip': 'The system default IP segment',
  'name': 'Admin',
  'parentId': '-300',
  'percent': '0.0',
  'percentStr': '0.0%'},
 {'description': 'Management IP Segment',
  'endIp': '10.10.254.254',
  'id': '44',
  'ip': '10.10.0.1-10.10.254.254',
  'name': 'cyoung',
  'operatorGroup': ['http://kontrolissues.thruhere.net:8086/imcrs/plat/operatorGroup/1',
   'http://kontrolissues.thruhere.net:8086/imcrs/plat/operatorGroup/3',
   'http://kontrolissues.thruhere.net:8086/imcrs/plat/operatorGroup/2'],
  'parentId': '-300',
  'percent': '9.191458071632097E-5',
  'percentStr': '0.00%',
  'startIp': '10.10.0.1'},
 {'assignedIpScope': [{'description': 'Mobile First Management Subnet',
    'endIp': '10.11.10.254',
    'id': '168',
    'ip': '10.11.10.1-10.11.10.254',
    'name': 'cyoung',
    'parentId': '45',
    'percent': '0.003937007874015748',
    'percentStr': '0.39%',
    'rightIpId': '45',
    'startIp': '10.11.10.1'},
   {'description': 'Mobile First User Subnet',
    'endIp': '10.11.1.254',
    'id': '169',
    'ip': '10.11.1.1-10.11.1.254',
    'name': 'cyoung',
    'parentId': '45',
    'percent': '0.0',
    'percentStr': '0.0%',
    'rightIpId': '45',
    'startIp': '10.11.1.1'},
   {'description': 'Mobile First Server Subnet',
    'endIp': '10.11.20.254',
    'id': '170',
    'ip': '10.11.20.1-10.11.20.254',
    'name': 'cyoung',
    'parentId': '45',
    'percent': '0.0',
    'percentStr': '0.0%',
    'rightIpId': '45',
    'startIp': '10.11.20.1'}],
  'description': 'Mobile First Branch',
  'endIp': '10.11.254.254',
  'id': '45',
  'ip': '10.11.0.1-10.11.254.254',
  'name': 'cyoung',
  'operatorGroup': ['http://kontrolissues.thruhere.net:8086/imcrs/plat/operatorGroup/1',
   'http://kontrolissues.thruhere.net:8086/imcrs/plat/operatorGroup/3',
   'http://kontrolissues.thruhere.net:8086/imcrs/plat/operatorGroup/2'],
  'parentId': '-300',
  'percent': '1.5319096786053493E-5',
  'percentStr': '0.00%',
  'startIp': '10.11.0.1'},
 {'description': 'Cisco Branch',
  'endIp': '10.12.254.254',
  'id': '46',
  'ip': '10.12.0.1-10.12.254.254',
  'name': 'cyoung',
  'operatorGroup': ['http://kontrolissues.thruhere.net:8086/imcrs/plat/operatorGroup/2',
   'http://kontrolissues.thruhere.net:8086/imcrs/plat/operatorGroup/3',
   'http://kontrolissues.thruhere.net:8086/imcrs/plat/operatorGroup/1'],
  'parentId': '-300',
  'percent': '3.0638193572106986E-5',
  'percentStr': '0.00%',
  'startIp': '10.12.0.1'},
 {'description': 'DataCenter',
  'endIp': '10.20.254.254',
  'id': '47',
  'ip': '10.20.0.1-10.20.254.254',
  'name': 'cyoung',
  'operatorGroup': ['http://kontrolissues.thruhere.net:8086/imcrs/plat/operatorGroup/2',
   'http://kontrolissues.thruhere.net:8086/imcrs/plat/operatorGroup/3',
   'http://kontrolissues.thruhere.net:8086/imcrs/plat/operatorGroup/1'],
  'parentId': '-300',
  'percent': '1.5319096786053493E-5',
  'percentStr': '0.00%',
  'startIp': '10.20.0.1'},
 {'assignedIpScope': [{'description': 'default production segment',
    'endIp': '10.101.0.254',
    'id': '67',
    'ip': '10.101.0.1-10.101.0.254',
    'name': 'cyoung',
    'parentId': '48',
    'percent': '0.41732283464566927',
    'percentStr': '41.73%',
    'rightIpId': '48',
    'startIp': '10.101.0.1'},
   {'description': 'Phone IP Segment',
    'endIp': '10.101.16.254',
    'id': '69',
    'ip': '10.101.16.1-10.101.16.254',
    'name': 'cyoung',
    'parentId': '48',
    'percent': '0.003937007874015748',
    'percentStr': '0.39%',
    'rightIpId': '48',
    'startIp': '10.101.16.1'},
   {'description': 'test child func',
    'endIp': '10.101.17.254',
    'id': '70',
    'ip': '10.101.17.1-10.101.17.254',
    'name': 'cyoung',
    'parentId': '48',
    'percent': '0.0',
    'percentStr': '0.0%',
    'rightIpId': '48',
    'startIp': '10.101.17.1'},
   {'description': 'test range 2',
    'endIp': '10.101.18.24',
    'id': '71',
    'ip': '10.101.18.1-10.101.18.24',
    'name': 'cyoung',
    'parentId': '48',
    'percent': '0.0',
    'percentStr': '0.0%',
    'rightIpId': '48',
    'startIp': '10.101.18.1'}],
  'description': 'Main Branch',
  'endIp': '10.101.254.254',
  'id': '48',
  'ip': '10.101.0.1-10.101.254.254',
  'name': 'cyoung',
  'operatorGroup': ['http://kontrolissues.thruhere.net:8086/imcrs/plat/operatorGroup/1',
   'http://kontrolissues.thruhere.net:8086/imcrs/plat/operatorGroup/2',
   'http://kontrolissues.thruhere.net:8086/imcrs/plat/operatorGroup/3'],
  'parentId': '-300',
  'percent': '0.001669781549679831',
  'percentStr': '0.16%',
  'startIp': '10.101.0.1'},
 {'description': 'Juniper Branch',
  'endIp': '10.13.254.254',
  'id': '165',
  'ip': '10.13.0.1-10.13.254.254',
  'name': 'cyoung',
  'operatorGroup': ['http://kontrolissues.thruhere.net:8086/imcrs/plat/operatorGroup/1',
   'http://kontrolissues.thruhere.net:8086/imcrs/plat/operatorGroup/3',
   'http://kontrolissues.thruhere.net:8086/imcrs/plat/operatorGroup/2'],
  'parentId': '-300',
  'percent': '0.0',
  'percentStr': '0.0%',
  'startIp': '10.13.0.1'},
 {'description': 'Comware5 Branch',
  'endIp': '10.14.254.254',
  'id': '166',
  'ip': '10.14.0.1-10.14.254.254',
  'name': 'cyoung',
  'operatorGroup': ['http://kontrolissues.thruhere.net:8086/imcrs/plat/operatorGroup/2',
   'http://kontrolissues.thruhere.net:8086/imcrs/plat/operatorGroup/3',
   'http://kontrolissues.thruhere.net:8086/imcrs/plat/operatorGroup/1'],
  'parentId': '-300',
  'percent': '0.0',
  'percentStr': '0.0%',
  'startIp': '10.14.0.1'},
 {'assignedIpScope': [{'description': 'P2P Link between Cloud First Datacenter and Mobile First Branch',
    'endIp': '172.16.0.2',
    'id': '171',
    'ip': '172.16.0.1-172.16.0.2',
    'name': 'cyoung',
    'parentId': '167',
    'percent': '0.0',
    'percentStr': '0.0%',
    'rightIpId': '167',
    'startIp': '172.16.0.1'},
   {'description': 'P2P Link between Cloud First Data Center and Juniper Branch',
    'endIp': '172.16.0.6',
    'id': '172',
    'ip': '172.16.0.5-172.16.0.6',
    'name': 'cyoung',
    'parentId': '167',
    'percent': '0.0',
    'percentStr': '0.0%',
    'rightIpId': '167',
    'startIp': '172.16.0.5'},
   {'description': 'P2P Link between Cloud First Datacenter and HPE FlexFabric Branch',
    'endIp': '172.16.0.10',
    'id': '173',
    'ip': '172.16.0.9-172.16.0.10',
    'name': 'cyoung',
    'parentId': '167',
    'percent': '0.0',
    'percentStr': '0.0%',
    'rightIpId': '167',
    'startIp': '172.16.0.9'},
   {'description': 'P2P Link between Cloud First Datacenter and Cisco Branch',
    'endIp': '172.16.0.14',
    'id': '174',
    'ip': '172.16.0.13-172.16.0.14',
    'name': 'cyoung',
    'parentId': '167',
    'percent': '0.0',
    'percentStr': '0.0%',
    'rightIpId': '167',
    'startIp': '172.16.0.13'}],
  'description': 'WAN P2P Links',
  'endIp': '172.16.30.254',
  'id': '167',
  'ip': '172.16.0.0-172.16.30.254',
  'name': 'cyoung',
  'operatorGroup': ['http://kontrolissues.thruhere.net:8086/imcrs/plat/operatorGroup/2',
   'http://kontrolissues.thruhere.net:8086/imcrs/plat/operatorGroup/3',
   'http://kontrolissues.thruhere.net:8086/imcrs/plat/operatorGroup/1'],
  'parentId': '-300',
  'percent': '0.0',
  'percentStr': '0.0%',
  'startIp': '172.16.0.0'}]

    """
    if scopeId is None:
        get_ip_scope_url = "/imcrs/res/access/assignedIpScope"
    else:
        get_ip_scope_url = "/imcrs/res/access/assignedIpScope/ip?ipScopeId="+str(scopeId)

    f_url = url + get_ip_scope_url
    r = requests.get(f_url, auth=auth, headers=HEADERS)  # creates the URL using the payload variable as the contents
    try:
        if r.status_code == 200:
            ipscopelist = (json.loads(r.text))
            return ipscopelist['assignedIpScope']
    except requests.exceptions.RequestException as e:
            return "Error:\n" + str(e) + " get_ip_scope: An Error has occured"

def get_ip_scope_detail(scopeId, auth, url ):
    """
    function requires no inputs and returns all IP address scopes currently configured on the HPE IMC server. If the
    optional scopeId parameter is included, this will automatically return only the desired scope id.
    :param scopeId: integer of the desired scope id ( optional )

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return: dictionary, may containing multiple entries if sub-scopes have been created

    :rtype: dict

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.termaccess import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> get_ip_scope_detail('45', auth.creds, auth.url)
    {'assignedIpScope': [{'description': 'Mobile First Management Subnet',
   'endIp': '10.11.10.254',
   'id': '168',
   'ip': '10.11.10.1-10.11.10.254',
   'name': 'cyoung',
   'parentId': '45',
   'percent': '0.003937007874015748',
   'percentStr': '0.39%',
   'rightIpId': '45',
   'startIp': '10.11.10.1'},
  {'description': 'Mobile First User Subnet',
   'endIp': '10.11.1.254',
   'id': '169',
   'ip': '10.11.1.1-10.11.1.254',
   'name': 'cyoung',
   'parentId': '45',
   'percent': '0.0',
   'percentStr': '0.0%',
   'rightIpId': '45',
   'startIp': '10.11.1.1'},
  {'description': 'Mobile First Server Subnet',
   'endIp': '10.11.20.254',
   'id': '170',
   'ip': '10.11.20.1-10.11.20.254',
   'name': 'cyoung',
   'parentId': '45',
   'percent': '0.0',
   'percentStr': '0.0%',
   'rightIpId': '45',
   'startIp': '10.11.20.1'}],
 'description': 'Mobile First Branch',
 'endIp': '10.11.254.254',
 'id': '45',
 'ip': '10.11.0.1-10.11.254.254',
 'name': 'cyoung',
 'operatorGroup': ['http://kontrolissues.thruhere.net:8086/imcrs/plat/operatorGroup/1',
  'http://kontrolissues.thruhere.net:8086/imcrs/plat/operatorGroup/2',
  'http://kontrolissues.thruhere.net:8086/imcrs/plat/operatorGroup/3'],
 'parentId': '-300',
 'percent': '1.5319096786053493E-5',
 'percentStr': '0.00%',
 'startIp': '10.11.0.1'}

    """
    get_ip_scope_url = "/imcrs/res/access/assignedIpScope/"+str(scopeId)

    f_url = url + get_ip_scope_url
    r = requests.get(f_url, auth=auth, headers=HEADERS)  # creates the URL using the payload variable as the contents
    try:
        if r.status_code == 200:
            ipscopelist = (json.loads(r.text))
            return ipscopelist
    except requests.exceptions.RequestException as e:
            return "Error:\n" + str(e) + " get_ip_scope: An Error has occured"


def get_ip_scope_hosts( scopeId, auth, url):
    """
    Function requires input of scope ID and returns list of allocated IP address for the specified scope

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :param scopeId: Interger of teh desired scope id

    :return: list of dictionary objects where each element of the list represents a single host assigned to the IP scope

    :rtype: list

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.termaccess import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> get_ip_scope_hosts('45', auth.creds, auth.url)
    [{'description': 'HP MSR930',
  'id': '31',
  'ip': '10.11.10.1',
  'name': 'R1Br4Core',
  'parentId': '168'}]

    """
    get_ip_scope_url = "/imcrs/res/access/assignedIpScope/ip?size=10000&ipScopeId="+str(scopeId)
    f_url = url + get_ip_scope_url
    r = requests.get(f_url, auth=auth, headers=HEADERS)  # creates the URL using the payload variable as the contents
    try:
        if r.status_code == 200:
            ipscopelist = (json.loads(r.text))
            ipscopelist = ipscopelist['assignedIpInfo']
            if type(ipscopelist) is dict:
                ipscope = []
                ipscope.append(ipscopelist)
                return ipscope
            return ipscopelist
    except requests.exceptions.RequestException as e:
            return "Error:\n" + str(e) + " get_ip_scope: An Error has occured"


def add_ip_scope(startIp, endIp, name, description, auth, url):
    """
    Function takes input of four strings Start Ip, endIp, name, and description to add new Ip Scope to terminal access
    in the HPE IMC base platform

    :param startIp: str Start of IP address scope ex. '10.101.0.1'

    :param endIp: str End of IP address scope ex. '10.101.0.254'

    :param name: str Name of the owner of this IP scope  ex. 'admin'

    :param description: str description of the Ip scope

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return: 200 if successfull

    :rtype:

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.termaccess import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> add_ip_scope('10.50.0.1', '10.50.0.254', 'cyoung', 'test group', auth.creds, auth.url)
    IP Scope Successfully Created
    200


    """
    if auth is None or url is None:  # checks to see if the imc credentials are already available
        set_imc_creds()

    add_ip_scope_url = "/imcrs/res/access/assignedIpScope"
    f_url = url + add_ip_scope_url
    payload = ('''{  "startIp": "%s", "endIp": "%s","name": "%s","description": "%s" }'''
               %(str(startIp), str(endIp), str(name), str(description)))
    r = requests.post(f_url, auth=auth, headers=HEADERS, data=payload) # creates the URL using the payload variable as the contents
    try:
        if r.status_code == 200:
            print("IP Scope Successfully Created")
            return r.status_code
        elif r.status_code == 409:
            print ("IP Scope Already Exists")
            return r.status_code
    except requests.exceptions.RequestException as e:
            return "Error:\n" + str(e) + " add_ip_scope: An Error has occured"

def add_child_ip_scope(startIp, endIp, name, description, scopeid, auth, url):
    """
    Function takes input of four strings Start Ip, endIp, name, and description to add new Ip Scope to terminal access
    in the HPE IMC base platform

    :param startIp: str Start of IP address scope ex. '10.101.0.1'

    :param endIp: str End of IP address scope ex. '10.101.0.254'

    :param name: str Name of the owner of this IP scope  ex. 'admin'

    :param description: str description of the Ip scope

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return: 200

    :rtype:

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.termaccess import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> add_child_ip_scope('10.50.0.1', '10.50.0.126', 'cyoung', 'test sub scope', '175', auth.creds, auth.url)
    IP Scope Successfully Created
    200


    """
    if auth is None or url is None:  # checks to see if the imc credentials are already available
        set_imc_creds()

    add_ip_scope_url = "/imcrs/res/access/assignedIpScope/" + str(scopeid)
    f_url = url + add_ip_scope_url
    payload = ('''{  "startIp": "%s", "endIp": "%s","name": "%s","description": "%s", "parentId" : "%s"}'''
               %(str(startIp), str(endIp), str(name), str(description), str(scopeid)))
    r = requests.post(f_url, auth=auth, headers=HEADERS, data=payload) # creates the URL using the payload variable as the contents
    try:
        if r.status_code == 200:
            print("IP Scope Successfully Created")
            return r.status_code
        elif r.status_code == 409:
            print ("Conflict with Current Scope")
            return r.status_code
    except requests.exceptions.RequestException as e:
            return "Error:\n" + str(e) + " add_ip_scope: An Error has occured"

def add_scope_ip(ipaddress, name, description, scopeid, auth, url):
    """
    Function to add new host IP address allocation to existing scope ID

    :param ipaddress:

    :param name: name of the owner of this host

    :param description: Description of the host

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return:

    :rtype:

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.termaccess import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> add_scope_ip('10.50.0.5', 'cyoung', 'New Test Host','175', auth.creds, auth.url)

    """
    new_ip = { "ip": ipaddress,
      "name": name,
      "description": description}
    add_scope_ip_url = '/imcrs/res/access/assignedIpScope/ip?ipScopeId='+str(scopeid)
    f_url = url + add_scope_ip_url
    payload = json.dumps(new_ip)
    r = requests.post(f_url, auth=auth, headers=HEADERS,
                      data=payload)  # creates the URL using the payload variable as the contents
    try:
        if r.status_code == 200:
            print("IP Scope Successfully Created")
            return r.status_code
        elif r.status_code == 409:
            print("IP Scope Already Exists")
            return r.status_code
    except requests.exceptions.RequestException as e:
        return "Error:\n" + str(e) + " add_ip_scope: An Error has occured"

def remove_scope_ip(hostid, auth, url):
    """
    Function to add new IP address allocation to existing scope ID

    :param hostid: Host id of the host to be deleted

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return: String of HTTP response code. Should be 204 is successfull

    :rtype: str

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.termaccess import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> remove_scope_ip('177', auth.creds, auth.url)
    204

    """
    add_scope_ip_url = '/imcrs/res/access/assignedIpScope/ip/'+str(hostid)
    f_url = url + add_scope_ip_url

    r = requests.delete(f_url, auth=auth, headers=HEADERS,
                      )
    try:
        if r.status_code == 204:
            print("IP Scope Successfully Created")
            return r.status_code
        elif r.status_code == 409:
            print("IP Scope Already Exists")
            return r.status_code
    except requests.exceptions.RequestException as e:
        return "Error:\n" + str(e) + " add_ip_scope: An Error has occured"



def get_scope_id(network_address, auth, url):
    """

    :param network_address: network address of the target scope in format x.x.x.x/yy  where x.x.x.x representents the
    network address and yy represents the length of the subnet mask.  Example:  10.50.0.0 255.255.255.0 would be written
    as 10.50.0.0/24

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return: str object which contains the numerical ID of the target scope

    :rtype: str


    >>>get_scope_id('10.50.0.0/24', auth.creds, auth.url)
    '175'

    """
    netaddr = ipaddress.ip_network(network_address)
    scopes = get_ip_scope(auth, url)
    for i in scopes:
        if int(i['id']) > 0:
            if ipaddress.ip_address(i['startIp']) in netaddr and ipaddress.ip_address(i['endIp']) in netaddr:
                return i['id']
            if "assignedIpScope" in i:
                for child in i['assignedIpScope']:
                    if ipaddress.ip_address(child['startIp']) in netaddr and ipaddress.ip_address(child['endIp']) in netaddr:

                        return child['id']









    #Add host to IP scope
    #http://10.101.0.203:8080/imcrs/res/access/assignedIpScope/ip?ipScopeId=1
    '''{
      "ip": "10.101.0.1",
      "name": "Cisco2811.lab.local",
      "description": "Cisco 2811",
      "parentId": "18"
    }'''


#!/usr/bin/env python3
# author: @netmanchris



# This section imports required libraries
import json
import requests



HEADERS = {'Accept': 'application/json', 'Content-Type':
    'application/json', 'Accept-encoding': 'application/json'}

def get_client_info_all(auth, url):
    """
    function takes no input to RESTFUL call to HP IMC

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return: list of dictionaries where each element of the list represents one client as discovered by the HPE IMC
    Wireless Services Management module.

    :rtype: list

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.wsm.clientinfo import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> get_client_info_all(auth.creds, auth.url)
    [{'acDevId': '31',
  'acIpAddress': '10.10.10.5',
  'acLabel': 'HP830_WSC',
  'apIpAddress': '10.101.0.107',
  'apLabel': 'msm525_ap3',
  'apMacAddress': '78:48:59:49:16:40',
  'apSerialId': 'CN48GTH031',
  'channel': '40',
  'ipAddress': '0.0.0.0',
  'location': 'UWW',
  'mac': '10:40:f3:ee:f0:be',
  'radioType': '32',
  'signalStrength': '-76',
  'ssid': 'famyoung',
  'upTime': '2551',
  'userName': ''},
 {'acDevId': '31',
  'acIpAddress': '10.10.10.5',
  'acLabel': 'HP830_WSC',
  'apIpAddress': '10.101.0.105',
  'apLabel': 'msm525_ap4',
  'apMacAddress': '78:48:59:49:18:C0',
  'apSerialId': 'CN48GTH03G',
  'channel': '11',
  'ipAddress': '0.0.0.0',
  'location': 'UWW',
  'mac': '24:0a:64:6d:55:79',
  'radioType': '16',
  'signalStrength': '-71',
  'ssid': 'famyoung',
  'upTime': '402790',
  'userName': ''},
 {'acDevId': '31',
  'acIpAddress': '10.10.10.5',
  'acLabel': 'HP830_WSC',
  'apIpAddress': '10.101.0.107',
  'apLabel': 'msm525_ap3',
  'apMacAddress': '78:48:59:49:16:40',
  'apSerialId': 'CN48GTH031',
  'channel': '1',
  'ipAddress': '0.0.0.0',
  'location': 'UWW',
  'mac': '3c:4a:92:5b:c2:43',
  'radioType': '16',
  'signalStrength': '-43',
  'ssid': 'famyoung',
  'upTime': '422422',
  'userName': ''},
 {'acDevId': '31',
  'acIpAddress': '10.10.10.5',
  'acLabel': 'HP830_WSC',
  'apIpAddress': '10.101.0.105',
  'apLabel': 'msm525_ap4',
  'apMacAddress': '78:48:59:49:18:C0',
  'apSerialId': 'CN48GTH03G',
  'channel': '48',
  'ipAddress': '0.0.0.0',
  'location': 'UWW',
  'mac': 'b0:34:95:3f:3c:01',
  'radioType': '32',
  'signalStrength': '-48',
  'ssid': 'famyoung',
  'upTime': '404164',
  'userName': ''},
 {'acDevId': '31',
  'acIpAddress': '10.10.10.5',
  'acLabel': 'HP830_WSC',
  'apIpAddress': '10.101.0.105',
  'apLabel': 'msm525_ap4',
  'apMacAddress': '78:48:59:49:18:C0',
  'apSerialId': 'CN48GTH03G',
  'channel': '11',
  'ipAddress': '0.0.0.0',
  'location': 'UWW',
  'mac': 'f4:b8:5e:00:e6:45',
  'radioType': '16',
  'signalStrength': '-82',
  'ssid': 'famyoung',
  'upTime': '335436',
  'userName': ''}]
    """
    get_client_info_all_url = "/imcrs/wlan/clientInfo/queryAllClientBasicInfo"
    f_url = url + get_client_info_all_url
    payload = None
    r = requests.get(f_url, auth=auth,
                     headers=HEADERS)  # creates the URL using the payload variable as the contents
    # print(r.status_code)
    try:
        if r.status_code == 200:
            if len(r.text) > 0:
                return json.loads(r.text)['clientBasicInfo']
    except requests.exceptions.RequestException as e:
            return "Error:\n" + str(e) + " get_client_info_all: An Error has occured"


def get_client_online_history_all(auth, url):
    """
    function takes no input to RESTFUL call to HP IMC
:param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return: list of dictionaries where each element of the list represents one client as discovered by the HPE IMC
    Wireless Services Management module.

    :rtype: list

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.wsm.clientinfo import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> get_client_online_history_all(auth.creds, auth.url)
    [{'acDevId': '31',
  'apName': 'msm525_ap3',
  'apSerialId': 'CN48GTH031',
  'authenMode': '2',
  'channel': '36',
  'ciphers': '16',
  'faultTime': '2016-06-21 12:15:13',
  'ip': '0.0.0.0',
  'loginTime': '2016-06-21 12:03:13',
  'mac': '00:23:12:0c:24:9f',
  'onlineTime': '00:12:00',
  'position': 'UWW',
  'radioId': '1',
  'radioType': '32',
  'rxByte': '32',
  'rxNoise': '-104',
  'rxSnr': '29',
  'singalStrength': '-66',
  'ssid': 'famyoung',
  'txByte': '3',
  'userName': ''},
 {'acDevId': '31',
  'apName': 'msm525_ap3',
  'apSerialId': 'CN48GTH031',
  'authenMode': '2',
  'channel': '36',
  'ciphers': '16',
  'faultTime': '2016-06-21 15:06:13',
  'ip': '0.0.0.0',
  'loginTime': '2016-06-21 14:42:13',
  'mac': '00:23:12:0c:24:9f',
  'onlineTime': '00:24:00',
  'position': 'UWW',
  'radioId': '1',
  'radioType': '32',
  'rxByte': '37',
  'rxNoise': '-103',
  'rxSnr': '28',
  'singalStrength': '-67',
  'ssid': 'famyoung',
  'txByte': '3',
  'userName': ''}]

    """
    get_client_online_history_all_url = "/imcrs/wlan/clientInfo/queryClientOnlineHistoryInfo"
    f_url = url + get_client_online_history_all_url
    payload = None
    r = requests.get(f_url, auth=auth,
                     headers=HEADERS)  # creates the URL using the payload variable as the contents
    # print(r.status_code)
    try:
        if r.status_code == 200:
            if len(r.text) > 0:
                return json.loads(r.text)['clientOnlineHistoryInfo']
    except requests.exceptions.RequestException as e:
            return "Error:\n" + str(e) + " get_client_online_history_all: An Error has occured"